
import javassist.*;

public class Example5 {

	public static void main (String[] args) throws Exception {
		// did we get one or two parameters?
		boolean gotTwoParameters = args.length > 2;

		ClassPool pool = ClassPool.getDefault();
		CtClass exprClass = pool.get("Expr");

		// the class name is 'Eval'
		CtClass evalClass = pool.makeClass("Eval"+System.currentTimeMillis());
			// Since we don't really care about the class name, we'll append the
			// current milliseconds as a minimal guard against multi-threading problems

		// extend the 'Expr" class, so we can access all the Math functions
		evalClass.setSuperclass(exprClass);

		// the interface implemented by the new class
		evalClass.setInterfaces(
			new CtClass[] { pool.makeClass("Evaluator2") });

		// add the eval method to the class
		// return NaN if two parameters were passed in
		evalClass.addMethod(
			CtNewMethod.make("public double eval (double x) { return ("
					+ (gotTwoParameters ? "Double.NaN" : args[0]) + ") ; }", evalClass));

		// and another eval method that takes two parameters
		evalClass.addMethod(
			CtNewMethod.make("public double eval (double x, double y) { return ("+args[0]+"); }", evalClass));

		// get a java.lang.Class object from Javassist's CtClass object
		Class clazz = evalClass.toClass();

		// instantiate it and cast it to Evaluator
		Evaluator2 obj  = (Evaluator2) clazz.newInstance();

		// and finally call the method
		double result = gotTwoParameters
						? obj.eval(Double.parseDouble(args[1]), Double.parseDouble(args[2]))
						: obj.eval(Double.parseDouble(args[1]));
		System.out.println(result);
	}
}

